#!/usr/bin/perl -I.
# Dreamcast Version
# Jet Grind Radio
# Graffiti Displayer
# 


use strict;

use CGI;
use	DC;
use POSIX	qw(strftime ceil);
use Image::Magick;
use DB;

## Set up variables
use vars	(	'$dc',	# DC object
				'@IN', '@BINDATA', '@oldDATA', '$allrank', '$todayrank',
				'$errorURL', '$cgiPath', '$date', '$cookieDate',
				'$root', '$servername', '$path', '$serverfilename', '$cgidir',
				'@SizeTbl', '@RankTbl', '%graffiti', 
				'$Type', '$type', '$Rank', '$Size', '$Mode', '$sort', '$sortby', '$title',
				'$RankLow', '$RankHigh', '$adminkey', '$pending',
				'$MAX', '$START', '$END', '$UserKey', '$searchID', '$MaxRec', '$nextflag',
			);


$MAX = 10;			# Default number of menu choices

# Stage Related Data
@SizeTbl = (
		# key => [ Graffiti Size Name, Graffiti Graphic, VMU File ]
		[ "Small",			"/images/small.jpg",	"JETSET___SMA"],
		[ "Large",			"/images/large.jpg",	"JETSET___LAR"],
		[ "Extra-Large",	"/images/ex_large.jpg", "JETSET___XLA"],
        );

@RankTbl = (
			# key => [ Class Name, Class Graphic ]
			[ "N/A",		"/images/rank_na.gif" ],
			[ "Pedal!",		"/images/rank_pedal.gif" ],
			[ "Motor!",		"/images/rank_motor.gif" ],
			[ "Engine!",	"/images/rank_engine.gif" ],
			[ "Turbo!",		"/images/rank_turbo.gif" ],
			[ "Nitro!",		"/images/rank_nitro.gif" ],
			[ "Jet!",		"/images/rank_jet.gif" ],
		 );


###############################################################


# Handling time
	$date = strftime("%m/%d/%Y, %I:%M %p (%Z)", localtime);
	$cookieDate = strftime("Date: %a, %d %b %Y %H:%M:%S %Z", gmtime);

	my $q = new CGI;
	$dc = new DC($q);
	$dc->q->import_names('IN');

	$errorURL = "graffiti.html";
	$cgiPath = "/cgi-bin/jetsetradio/graffiti.cgi";
	$root = "../../";
	$servername = "tmp";
	$cgidir = "cgi-bin/jetsetradio";
	$path = $root.$servername;  # path
	$serverfilename = sprintf "%08d", $$;

	my $userid = $dc->q->cookie(-name=>'login_userID');

	# The default user does not have administration rights		
	$adminkey = "";
	$pending = 'N';
		
	@BINDATA = ();
	$nextflag = 1;
	$Type = 0;
	$title = "";

	if($dc->q->param('command') eq "")
	{
		&check_data(1);		# First time through
	} else {
		&check_data(0);
	}
    
    if(($Type >= 0) && ($Type <= 3)) {
		if ($dc->q->param('command') eq 'menu')
		{
			my $cookie = $dc->q->cookie(-name => 'graffiti',
									-value   => [$dc->q->param('SIZE'),
												 $dc->q->param('MODE')],
									-expires => '+7d',
									-path    => '/');

			print "Set-Cookie: ".$cookie."\n";

			print $cookieDate."\n";

			# Cache control does not work with DreamKey
#			print "Cache-control: no-cache\n\n";

			# Menu template
			$dc->output_form("html/graffiti_menu.html");
		} elsif($dc->q->param('command') eq 'form') {
			# Form output again
			$dc->output_form("html/graffiti_form.html");
		} elsif($dc->q->param('command') eq "") {
			# Form output again
			$dc->output_form("html/graffiti_form.html");
		} elsif ($dc->q->param('command') eq 'search') {
			&check_search;
			# Form output again
			$dc->output_form("html/graffiti_search.html");
		} elsif ($dc->q->param('command') eq 'view') {
			&check_search;
			&output_records(&read_data());
		} elsif ($dc->q->param('command') eq 'display') {
			# Output the Graffiti Display Page
			&check_entry;

			$dc->output_form("html/graffiti_select.html");
		} elsif ($dc->q->param('command') eq 'verify') {
			# Record data into Database
			my ($dbh, $errN, $errstr, @row, $key);
			
			# Check to make sure that Login returned a valid userID
			if ($userid eq "")
			{
				$dc->error("[graffiti] Error logging in.  Please try again.", "/home.html");
			}

			# Read user information from the table
			$key = $dc->make_key($userid);
			
			($dbh, $errstr) = DB::openOracleConnection();
			if( !$dbh )
			{
				$dc->error("[graffiti] Error connecting to database: $errstr");
			}
			
			# Update user table information
			($errN, $errstr, @row) = DB::getRow_UserByUK( $dbh, $dc->DBaseTable('USER'), $key );
			if ($errN)
			{
				$dc->error("[graffiti] Fetching user key: $errN .... $errstr", $errorURL);
			}
				
			($errN, $errstr)       = DB::modifyRow_UserByUK( $dbh, $dc->DBaseTable('USER'), $key, $row[$dc->UserOff('EMAIL')], $row[$dc->UserOff('NAME')], $row[$dc->UserOff('CITY')], $row[$dc->UserOff('STATE')], $row[$dc->UserOff('COUNTRY')], $dc->q->param('MAIL'), $row[$dc->UserOff('ADMIN')], $row[$dc->UserOff('GOLD')], $row[$dc->UserOff('SILVER')], $row[$dc->UserOff('BRONZE')], $row[$dc->UserOff('BLUERIB')], time );
			if ($errN)
			{
				$dc->error("[graffiti] Unable to update user info: $errN .... $errstr", $errorURL);
			}
				
			DB::closeOracleConnection( $dbh );

			# Enter the data into the Graffiti Database
			my $added = new_record($userid, $Size);

			# Display Record
			&output_verify($added);
		}
	}

exit(0);

#--------------------------------------------------------
# Data Check
#--------------------------------------------------------
sub check_data
{
	my $first = shift;

	my $MAIL = "";
    my $Warning = "";
	my ($dbh, $errN, $errstr, @row, $key, $val);

	if($dc->q->param('command') eq "")
	{
		my @level = $dc->q->cookie(-name=>'graffiti');
		if (($level[0] eq "") || ($level[1] eq ""))
		{
			$dc->error("Cookie of \"graffiti\" has not been set.<BR>Please make sure your Dreamcast clock is correct, or that Cookies are enabled.", $errorURL);
		}

		$dc->q->param('SIZE', $level[0]);
		$dc->q->param('MODE', $level[1]);
	}

    $Size = $dc->q->param('SIZE');
	$Mode = $dc->q->param('MODE');
    if(!defined($SizeTbl[$Size]->[0])) {
        $dc->error("[graffiti] Invalid graffiti size was chosen.<BR>Please select a different graffiti size.", $errorURL);
    }

	$Type = 0;				# Default to Small Size
	# Story Mode
    if ($Size == 0) {
		$dc->q->param('SizeText', $SizeTbl[$Size]->[0]);
		$dc->q->param('SizeImg', $SizeTbl[$Size]->[1]);
		$dc->q->param('VMUFile', $SizeTbl[$Size]->[2]);
    } elsif($Size == 1) {			# Jet Crush Mode
		$dc->q->param('SizeText', $SizeTbl[$Size]->[0]);
		$dc->q->param('SizeImg', $SizeTbl[$Size]->[1]);
		$dc->q->param('VMUFile', $SizeTbl[$Size]->[2]);
    } elsif($Size == 2) {		# Jet Technique Mode
		$dc->q->param('SizeText', $SizeTbl[$Size]->[0]);
		$dc->q->param('SizeImg', $SizeTbl[$Size]->[1]);
		$dc->q->param('VMUFile', $SizeTbl[$Size]->[2]);
	}

	if (($IN::command eq "") || ($IN::command eq "form") || ($IN::command eq "verify"))
	{
		# Don't bother processing if just viewing the World Rankings

		# Data Decode
		if( &decode($dc->q->param('bindata')) != 0) {
			$dc->error("Your graffiti data was lost.<BR>Please try again and reupload your ranking file.", $errorURL);
		}

		if ($first == 1)
		{
			# Read user information from the table
			if ($userid ne "")
			{
				($dbh, $errstr) = DB::openOracleConnection();
				if( !$dbh )
				{
					$dc->error("[graffiti] Error connecting to database: $errstr", $errorURL);
				}

				($errN, $errstr, @row) = DB::getRow_UserByUK( $dbh, $dc->DBaseTable('USER'), $userid );
				if ($errN)
				{
					$dc->error("[graffiti] Fetching user key: $errN .... $errstr", $errorURL);
				}

				DB::closeOracleConnection( $dbh );

				$dc->q->param('NAME', $row[$dc->UserOff('NAME')]);		# Name
				$dc->q->param('CITY', $row[$dc->UserOff('CITY')]);		# City
				$dc->q->param('STATE', $row[$dc->UserOff('STATE')]);
				$dc->q->param('COUNTRY', $row[$dc->UserOff('COUNTRY')]);
				$dc->q->param('state', $dc->state_name($row[$dc->UserOff('STATE')]));	# State
				$dc->q->param('country', $dc->country_name($row[$dc->UserOff('COUNTRY')]));# Country
				$dc->q->param('MAIL', $row[$dc->UserOff('MFLAG')]);		# Message Flag
			}
			else
			{
				$dc->error("[graffiti] Error - No user has logged in.  Please try again.", "home.html");
			}

			# Set default to No
			if ($dc->q->param('MAIL') ne 'Y')
			{
				$dc->q->param('MAIL', 'N');
			}
		}
		else
		{
			$dc->q->param('NAME', $dc->q->param('NAME'));

			$dc->q->param('CITY', $dc->smut_check($IN::CITY));

			$dc->q->param('STATE', $IN::STATE);

			$dc->q->param('COUNTRY', $IN::COUNTRY);

			$dc->q->param('state', $dc->state_name($IN::STATE));

			$dc->q->param('country', $dc->country_name($IN::COUNTRY));

			if($IN::MAIL ne "Y") {
				$dc->q->param('MAIL','N');
			}
		}

		my $trunc = substr($dc->q->param('COMMENT'),0,127);
		$dc->q->param('COMMENT', $dc->smut_check($trunc));

		# Set up the error message
		$dc->q->param('Warning', $Warning);
	}
}

#--------------------------------------------------------
# Additional New Data
#--------------------------------------------------------
sub new_record {
	my $UserKey = shift;
	my $Size = shift;

	# Record data into Database
	my ($dbh, $errN, $errstr, @row, $newentry, $binary);

	# Initially disallow record change					
	$newentry = 0;

	($dbh, $errstr) = DB::openOracleConnection();
	if( !$dbh )
	{
		$dc->error("[graffiti] Error connecting to database: $errstr");
	}

	# Retrieve Awards
	($errN, $errstr, @row) = DB::getRow_UserByUK( $dbh, $dc->DBaseTable('USER'), $UserKey );
	if ($errN)
	{
		$dc->error("[graffiti] Fetching user Info: $errN .... $errstr", $errorURL);
	}
				
	DB::closeOracleConnection( $dbh );

	# Key Acquisition
	my $MAIL = $row[$dc->UserOff('EMAIL')];
	$MAIL =~ y/A-Z/a-z/;
	$dc->q->param('email', $MAIL);

	$dc->q->param('GOLD', $row[$dc->UserOff('GOLD')]);
	$dc->q->param('SILVER', $row[$dc->UserOff('SILVER')]);
	$dc->q->param('BRONZE', $row[$dc->UserOff('BRONZE')]);
	$dc->q->param('BLUERIB', $row[$dc->UserOff('BLUERIB')]);
				
	$dc->q->param('TITLE', $dc->smut_check($IN::TITLE));

	$dc->q->param('DESCR', $dc->smut_check($IN::DESCR));

	# Always a New Record for Graffiti so go ahead and store it
	$newentry = 1;

	# Change out the VMU Header for the appropriate one for downloading
	my $headername = "";
	if ($Size == 0)
	{
		$headername = "icon_image_s.bin";
	}
	elsif ($Size == 1)
	{
		$headername = "icon_image_l.bin";
	}
	elsif ($Size == 2)
	{
		$headername = "icon_image_x.bin";
	}
	else
	{
		$dc->error("[graffiti] Invalid Graffiti size was chosen.", $errorURL);
	}
	
	# Read data for tournament header.
	if(open(FILE,"${root}/${cgidir}/${headername}"))
	{
		read(FILE,$graffiti{'header'}, 0x280);
		close(FILE);
	}
	else
	{
		$dc->error("[graffiti] Unable to load header file \"".$headername."\".", $errorURL);
	}

	# Create the data for attaching to the Record
	# Users Graffiti Title
	if (length($dc->q->param('TITLE')) != 0)
	{
		$graffiti{'title'} = $dc->q->param('TITLE');
		$graffiti{'titleSize'} = length($dc->q->param('TITLE'));
	}
	else
	{
		$dc->q->param('TITLE', $graffiti{'title'});
	}
		
	# Users Graffiti Comment
	if (length($dc->q->param('DESCR')) != 0)
	{
		$graffiti{'comment'} = $dc->q->param('DESCR');
		$graffiti{'commentSize'} = length($dc->q->param('DESCR'));
	}
	else
	{
		$graffiti{'comment'} = "Custom Graffiti.";
		$graffiti{'commentSize'} = length($graffiti{'comment'});
		$dc->q->param('DESCR', $graffiti{'comment'});
	}
	
	# Check if need to leave e-mail blank
	if ($dc->q->param('MAIL') eq 'Y')
	{	
		# Users e-mail Address if user wants it shown
		if (length($MAIL) > 0)
		{
			$graffiti{'mailSize'} = length($MAIL);
			$graffiti{'mail'} = $MAIL;
		}
		else
		{
			# Email was not given
			$graffiti{'mailSize'} = 0;
			$graffiti{'mail'} = "";
		}
	}
	else
	{
		# Email was not shown
		$graffiti{'mailSize'} = 0;
		$graffiti{'mail'} = "";
	}
		
	$binary = $dc->make_vmfile_graffiti( \%graffiti );

	# Create the temporary image graphic
	my $result = open( JPGFILE, ">${path}/${serverfilename}.jpg" );
	if( $result )
	{
		print JPGFILE $graffiti{'image'};
		close( JPGFILE );
		chmod(0666,"${path}/${serverfilename}.jpg");
	}
	else
	{
		$dc->error("[graffiti] Unable to create JPG file.<BR>Graffiti file may be damaged.<BR>Please try again. ", $errorURL);
	}
	
	# Write new graffiti information to Database
	($dbh, $errstr) = DB::openOracleConnection();
	if( !$dbh )
	{
		$dc->error("[graffiti] Error connecting to database: $errstr");
	}

	# Place new graphic into Database
	($errN, $errstr)       = DB::insertRow_Graffiti( $dbh, $dc->DBaseTable('GRAFFITI'), $UserKey, $Size, $dc->q->param('TITLE'), $dc->q->param('DESCR'), $dc->q->param('COMMENT'), 0, 0, 0, 0.0, 'Y', "", time, time, $binary );
	if ($errN)
	{
		$dc->error("[graffiti] Unable to create Graffiti info: $errN .... $errstr", $errorURL);
	}

	DB::closeOracleConnection( $dbh );

	return ($newentry);
}

#------------------------------------------------
# Determine what your ranking is
#------------------------------------------------
sub determine_rank {
	my $Size = shift;
	my $type = shift;

	my ($dbh, $sth, $errN, $errstr, @row, $key, $UserID);
	my $rank = 0;
	my $searchtime = 0;
			
	if ($type eq "TODAY")
	{
		$searchtime = (1*24*60*60);			# (24 hours in seconds)
	}
	else
	{
		# 5 Years in seconds
		$searchtime = (5*365*24*60*60);		# (5 Years in seconds)
	}

	# Access the database
	($dbh, $errstr) = DB::openOracleConnection();
	if( !$dbh )
	{
		$dc->error("[graffiti] Error connecting to database: $errstr");
	}

	# General search so use the different queries

	# Following code handles the various types of searches

	if ($IN::mailid ne "")
	{
		$UserID = $dc->get_key($IN::mailid);

		# Searching for a users name
		($errN, $errstr, $sth) = DB::getRow_GraffitiByUserIDandSize( $dbh, $dc->DBaseTable('GRAFFITI'), $UserID, $Size, $pending, "DESC", $searchtime );
		if ($errN)
		{
			$dc->error("[graffiti] Unable to Search for particular User Graffiti: $errN .... $errstr", $errorURL);
		}
	}
	else
	{
		if ($sort == 0)
		{
			if ($sortby == 0)
			{
				# Search by Ratings
				($errN, $errstr, $sth) = DB::getRow_GraffitiByRankandSize( $dbh, $dc->DBaseTable('GRAFFITI'), $RankLow, $RankHigh, $Size, $pending, "DESC", $searchtime );
				if ($errN)
				{
					$dc->error("[graffiti] Unable to get Graffiti Rank and Size Ascending Events: $errN .... $errstr", $errorURL);
				}
			}
			elsif ($sortby == 1)
			{
				# Just sort by Rank
				($errN, $errstr, $sth) = DB::getRow_GraffitiBySizeOrderbyRank( $dbh, $dc->DBaseTable('GRAFFITI'), $Size, $pending, "DESC", $searchtime );
				if ($errN)
				{
					$dc->error("[graffiti] Unable to get Graffiti DL Count and Size Descending Events: $errN .... $errstr", $errorURL);
				}
			}
			else
			{
				# Something went wrong so use the default search
				($errN, $errstr, $sth) = DB::getRow_GraffitiBySize( $dbh, $dc->DBaseTable('GRAFFITI'), $Size, $pending, "ASC", $searchtime );
				if ($errN)
				{
					$dc->error("[graffiti] Unable to get Graffiti Size Ascending Events: $errN .... $errstr", $errorURL);
				}
			}
		}
		elsif ($sort == 1)
		{
			# Search by Top Downloads
			($errN, $errstr, $sth) = DB::getRow_GraffitiByDLCountandSize( $dbh, $dc->DBaseTable('GRAFFITI'), $Size, $pending, "DESC", $searchtime );
			if ($errN)
			{
				$dc->error("[graffiti] Unable to get Graffiti DL Count and Size Descending Events: $errN .... $errstr", $errorURL);
			}
		}
		elsif ($sort == 2)
		{
			# Search by Newest Date
			if ($sortby == 0)
			{
				($errN, $errstr, $sth) = DB::getRow_GraffitiBySize( $dbh, $dc->DBaseTable('GRAFFITI'), $Size, $pending, "DESC", $searchtime );
				if ($errN)
				{
					$dc->error("[graffiti] Unable to get Graffiti Size and Ascending Events: $errN .... $errstr", $errorURL);
				}
			}
			# Search by Oldest Date
			elsif ($sortby == 1)
			{
				($errN, $errstr, $sth) = DB::getRow_GraffitiBySize( $dbh, $dc->DBaseTable('GRAFFITI'), $Size, $pending, "ASC", $searchtime );
				if ($errN)
				{
					$dc->error("[graffiti] Unable to get Graffiti Size and Descending Events: $errN .... $errstr", $errorURL);
				}
			}
			# Sort by Alphabetical Order
			elsif ($sortby == 2)
			{
				($errN, $errstr, $sth) = DB::getRow_GraffitiBySizeOrderbyTitle( $dbh, $dc->DBaseTable('GRAFFITI'), $Size, $pending, "ASC", $searchtime );
				if ($errN)
				{
					$dc->error("[graffiti] Unable to get Graffiti Title and Size Ascending Events: $errN .... $errstr", $errorURL);
				}
			}
			else
			{
				# Something went wrong so use the default search
				($errN, $errstr, $sth) = DB::getRow_GraffitiBySize( $dbh, $dc->DBaseTable('GRAFFITI'), $Size, $pending, "ASC", $searchtime );
				if ($errN)
				{
					$dc->error("[graffiti] Unable to get Graffiti Size Ascending Events: $errN .... $errstr", $errorURL);
				}
			}
		}
		elsif ($sort == 3)
		{
			# Search by Title
			($errN, $errstr, $sth) = DB::getRow_GraffitiSearchTitleandSize( $dbh, $dc->DBaseTable('GRAFFITI'), $title, $Size, $pending, "ASC", $searchtime );
			if ($errN)
			{
				$dc->error("[graffiti] Unable to get Graffiti Title and Size Ascending Events: $errN .... $errstr", $errorURL);
			}
		}
		else
		{
			# Something went wrong so use the default search
			($errN, $errstr, $sth) = DB::getRow_GraffitiBySize( $dbh, $dc->DBaseTable('GRAFFITI'), $Size, $pending, "ASC", $searchtime );
			if ($errN)
			{
				$dc->error("[graffiti] Unable to get Graffiti Size Ascending Events: $errN .... $errstr", $errorURL);
			}
		}
	}

	# Determine number of elements in search
    while( (@row) = DB::getNextRow_Graffiti($sth) )
	{
		++$rank;
	}

	DB::closeOracleConnection( $dbh );

	# If only displaying the 10 ten records then always return a maximum of ten
	if (($Mode >= 0) && ($Mode <= 3))
	{
		# Define that only 10 records will be shown
		if ($rank > 10)
		{
			# Only show top 10
			$rank = 10;
		}
	}

	# Side effect is total number of records have been read
	$MaxRec = $rank;


	return ($rank);
}

#------------------------------------------------
# Decode
#------------------------------------------------
sub decode {
    my $srcdata = shift;

	my $errorNum = 0;
	my($offset, $ptr, $pos, $length, $initials, $temp, $loop, $i, $j, $k, @tempArr);

	if ($srcdata eq "")
	{
		$dc->error("[graffiti] An error occurred while trying to decode your Graffiti data.  Please try uploading your Graffiti File again.", "world_rankings.html");
	}

    # BASE64Decode
    @BINDATA = $dc->decode_base64($srcdata);

	##############################################################
	# Data structure decode begin
		
	%graffiti = $dc->vmfile_graffiti(\@BINDATA);

    return($errorNum);
}

#--------------------------------------------------------
# Result Output
#--------------------------------------------------------
sub output_verify {
	my $newRecord = shift;

    print "Content-type: text/html\n\n";
    print <<EOF;

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">

<html>
<head>
	<title>Jet Grind Radio</title>
<meta name=x-uirequest content=nohscroll>
</head>

<body bgcolor=#E6CE00 x-marginleft=0 TEXT="#660000" LINK="#B96800" VLINK="#B96800" ALINK="#B96800" marginheight=0 marginwidth=0 topmargin=0 leftmargin=0>


<center>

<TABLE WIDTH="550" BORDER=0 CELLPADDING=0 CELLSPACING=0>
	<tr>
		<TD valign=top width=550><img src="/images/title_graffiti.gif" width=550 height=46 border=0 alt=""></td>
	</tr>
</table>
<TABLE WIDTH="550" BORDER=0 CELLPADDING=0 CELLSPACING=0>
	<tr>
		<TD width=7 valign=top><img src="/images/clear.gif" width=7 height=1 border=0 alt=""></td>
		<TD width=543 valign=top>
EOF

	print "Thank you for uploading your graffiti!<p>\n\n";

	print <<EOF;
		<br>
			<table width=100% cellpadding=6 cellspacing=0 border=0>
				<tr>
				
<!--BEGIN GRAFFITI TABLE-->
EOF

	print "<td bgcolor=#cccccc valign=top>".$dc->q->param('TITLE')." - ".$dc->q->param('DESCR');
	print "<BR>";

	if ($dc->Flags($dc->q->param('COUNTRY')) ne "")
	{
		print "<img src=\"/images/flags/".$dc->Flags($dc->q->param('COUNTRY'))."\" width=40 height=20 border=0><BR>";	
	} else {
		# Draw the generic flag
		print "<img src=\"/images/flags/flag_other.gif\" width=40 height=20 border=0><BR>";
	}

	print $dc->q->param('NAME')."<BR>\n";

	print $dc->q->param('CITY').", ";
	if ($dc->q->param('STATE') ne "") {
		printf "%s, ", $dc->q->param('state');
	}
	printf "%s<BR>\n", $dc->q->param('country');

	if ($dc->q->param('MAIL') eq "Y")
	{
		print $dc->q->param('email')."<BR>\n";
	} else {
		print "E-Mail Hidden<BR>\n";
	}

	print "</td>\n";
	print "<td bgcolor=#cccccc valign=top>".$date."<br><br>\n";

	print $dc->q->param('COMMENT');

	print <<EOF;
					</td>
				</tr>

				<tr>
					<td bgcolor=#cccccc valign=top colspan=2 align=center><img src="/tmp/${serverfilename}.jpg" border=0 alt=""></td>
				</tr>
				<tr>
					
					<td valign=top colspan=2 align=right><a href="/graffiti.html"><img src="/images/button_back.gif" width=200 height=30 border=1 alt="" align=right></a></td>
				</tr>
			</table><br>
			
		<br></td>

	</tr>
</table>

</center>
</body>

</html>
EOF

}

######################################################################
# Display contents of Graffiti
######################################################################

#--------------------------------------------------------
# Data Check
#--------------------------------------------------------
sub check_entry {
	my ($errN, $errstr, $dbh, $sth, @user, @graffiti, @admin);
	my ($location, $UniqKey, $ApprovalForm, $VoteForm, %graffitiInfo, @BINARY);
	my ($cookieName, $cookieVal, $rate, $total, $count, $average, $datestr, $err, $image);
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst);
	my ($str, $returnURL);
	
	$returnURL = "<form action=\"/cgi-bin/jetsetradio/graffiti.cgi\">\n";
	$returnURL .= "<INPUT TYPE=\"hidden\" NAME=\"SIZE\" VALUE=\"".$IN::SIZE."\">\n";
	$returnURL .= "<INPUT TYPE=\"hidden\" NAME=\"MODE\" VALUE=\"".$IN::MODE."\">\n";
	$returnURL .= "<INPUT TYPE=\"hidden\" NAME=\"mailid\" VALUE=\"".$IN::mailid."\">\n";
	$returnURL .= "<INPUT TYPE=\"hidden\" NAME=\"type\" VALUE=\"".$IN::type."\">\n";
	$returnURL .= "<INPUT TYPE=\"hidden\" NAME=\"sort\" VALUE=\"".$IN::sort."\">\n";
	$returnURL .= "<INPUT TYPE=\"hidden\" NAME=\"rating\" VALUE=\"".$IN::rating."\">\n";
	$returnURL .= "<INPUT TYPE=\"hidden\" NAME=\"sortby\" VALUE=\"".$IN::sortby."\">\n";
	$returnURL .= "<INPUT TYPE=\"hidden\" NAME=\"title\" VALUE=\"".$IN::title."\">\n";
	$returnURL .= "<INPUT TYPE=\"hidden\" NAME=\"start\" VALUE=\"".$IN::start."\">\n";
	$returnURL .= "<input type=\"hidden\" name=\"max\" value=\"".$IN::max."\">\n";
	$returnURL .= "<input type=\"hidden\" name=\"command\" value=\"view\">\n";
	$returnURL .= "<input type=\"image\" src=\"/images/button_back.gif\" width=200 height=30 border=1 alt=\"\" align=right>\n";
	$returnURL .= "</form>\n";

	$dc->q->param('returnURL', $returnURL);
	
	# Determine if user has Administration Rights
	# Read user information from the table
	if ($userid ne "")
	{
		($dbh, $errstr) = DB::openOracleConnection();
		if( !$dbh )
		{
			$dc->error("[graffiti] Error connecting to database: $errstr", $errorURL);
		}
		
		($errN, $errstr, @user) = DB::getRow_UserByUK( $dbh, $dc->DBaseTable('USER'), $userid );
		if ($errN)
		{
			$dc->error("[graffiti] Fetching user key: $errN .... $errstr", $errorURL);
		}
		
		DB::closeOracleConnection( $dbh );

		if ($user[$dc->UserOff('ADMIN')] eq 'Y')
		{
			$adminkey = $userid;
			$pending = 'Y';
		}
	}

	# Set up required parameters
	$UniqKey = $IN::UniqKey;
	$dc->q->param('UniqKey', $UniqKey);

	$rate = $IN::rate;

	# Login cookie is valid, so check to see if they have previously voted
	$cookieName = "graffiti_".$UniqKey;
	$cookieVal = $dc->q->cookie(-name=>$cookieName);

	#################################################
	# Read the information about the graffiti Piece
	($dbh, $errstr) = DB::openOracleConnection();
	if( !$dbh )
	{
		$dc->error("[graffiti] Error connecting to database: $errstr", $errorURL);
	}
	
	($errN, $errstr, $sth) = DB::getRow_GraffitiByUK( $dbh, $dc->DBaseTable('GRAFFITI'), $UniqKey );
	if ($errN)
	{
		$dc->error("[graffiti] Unable to locate the particular Graffiti record: $errN .... $errstr", $errorURL);
	}
	
	@graffiti = DB::getNextRow_Graffiti($sth);
	
	($errN, $errstr, @user) = DB::getRow_UserByUK( $dbh, $dc->DBaseTable('USER'), $graffiti[$dc->GraffitiOff('USERID')] );
	if ($errN)
	{
		$dc->error("[graffiti] Fetching user info for user \"".$graffiti[$dc->GraffitiOff('USERID')]."\": $errN .... $errstr", $errorURL);
	}
	
	DB::closeOracleConnection( $dbh );

	###############################################
	# Code to handle voting requirements
	if ($userid eq "")
	{
		# Voting Handling Code
		$VoteForm = <<"EOF";
					<b>Vote for this graffiti!</b><br><BR>
					
					Login with your Sega.com account, and vote for this artwork
EOF
	}
	else
	{
		if ($cookieVal eq "")
		{
			# Voting Handling Code
			$VoteForm = <<"EOF";
					<b>Vote for this graffiti!</b>
<form action="/cgi-bin/jetsetradio/graffiti.cgi">
<INPUT TYPE="hidden" NAME="UniqKey" VALUE="$UniqKey">
<INPUT TYPE="hidden" NAME="feature" VALUE="vote">
EOF

			$VoteForm .= "<INPUT TYPE=\"hidden\" NAME=\"SIZE\" VALUE=\"".$IN::SIZE."\">\n";
			$VoteForm .= "<INPUT TYPE=\"hidden\" NAME=\"MODE\" VALUE=\"".$IN::MODE."\">\n";
			$VoteForm .= "<INPUT TYPE=\"hidden\" NAME=\"mailid\" VALUE=\"".$IN::mailid."\">\n";
			$VoteForm .= "<INPUT TYPE=\"hidden\" NAME=\"type\" VALUE=\"".$IN::type."\">\n";
			$VoteForm .= "<INPUT TYPE=\"hidden\" NAME=\"sort\" VALUE=\"".$IN::sort."\">\n";
			$VoteForm .= "<INPUT TYPE=\"hidden\" NAME=\"rating\" VALUE=\"".$IN::rating."\">\n";
			$VoteForm .= "<INPUT TYPE=\"hidden\" NAME=\"sortby\" VALUE=\"".$IN::sortby."\">\n";
			$VoteForm .= "<INPUT TYPE=\"hidden\" NAME=\"title\" VALUE=\"".$IN::title."\">\n";
			$VoteForm .= "<INPUT TYPE=\"hidden\" NAME=\"start\" VALUE=\"".$IN::start."\">\n";
			$VoteForm .= "<INPUT TYPE=\"hidden\" NAME=\"max\" VALUE=\"".$IN::max."\">\n";

			$VoteForm .= "<select name=\"rate\" size=\"1\">";
			$VoteForm .= "<option value=\"1\" ".(($rate == 1) ? "SELECTED" : "").">1: Pedal</option>\n";
   			$VoteForm .= "<option value=\"2\" ".(($rate == 2) ? "SELECTED" : "").">2: Motor</option>\n";
   			$VoteForm .= "<option value=\"3\" ".(($rate == 3) ? "SELECTED" : "").">3: Engine</option>\n";
   			$VoteForm .= "<option value=\"4\" ".(($rate == 4) ? "SELECTED" : "").">4: Turbo</option>\n";
   			$VoteForm .= "<option value=\"5\" ".(($rate == 5) ? "SELECTED" : "").">5: Nitro</option>\n";
			$VoteForm .= "<option value=\"6\" ".(($rate == 6) ? "SELECTED" : "").">6: Jet</option></select>\n";

			$VoteForm .= <<"EOF";
<input type="hidden" name="command" value="display">
<input type=image src="/images/button_vote.gif" width=52 height=13 border=0 alt="">
</FORM>
EOF
		}
		else
		{
			# Voting Handling Code
			$VoteForm = <<"EOF";
					<b>Thank you for voting!</b><br><BR>
					
					Total number of votes for this artwork:&nbsp;&nbsp;<b>$graffiti[$dc->GraffitiOff('RANKCNT')]</b>
EOF
		}
}

	# Handle voting feature
	if (($IN::feature eq "vote") && ($cookieVal eq ""))
	{
		# Set up Voting Cookie
		my $cookie = $dc->q->cookie(-name => $cookieName,
									-value   => 'voted',
									-expires => '+1d',
									-path    => '/');

		print "Set-Cookie: ".$cookie."\n";

		print $cookieDate."\n";

		# Cache control does not work with DreamKey
#		print "Cache-control: no-cache\n\n";

		# Determine the new Average
		$total = $graffiti[$dc->GraffitiOff('RANKTOTAL')] + $rate;
		$count = $graffiti[$dc->GraffitiOff('RANKCNT')] + 1;
		$average = $total / $count;

		# Make the change to the Database
		($dbh, $errstr) = DB::openOracleConnection();
		if( !$dbh )
		{
			$dc->error("[graffiti] Error connecting to database: $errstr", $errorURL);
		}
	
		# Update the Voting Count Totals
		($errN, $errstr)       = DB::modifyCountRow_GraffitiByUK( $dbh, $dc->DBaseTable('GRAFFITI'), $UniqKey, $graffiti[$dc->GraffitiOff('DLCOUNT')], $total, $count, $average );
		if ($errN)
		{
			$dc->error("[graffiti] Updating Graffiti Average Count: $errN .... $errstr", $errorURL);
		}

		# Re-update the Graffiti Information
		($errN, $errstr, $sth) = DB::getRow_GraffitiByUK( $dbh, $dc->DBaseTable('GRAFFITI'), $UniqKey );
		if ($errN)
		{
			$dc->error("[graffiti] Unable to locate the particular Graffiti record: $errN .... $errstr", $errorURL);
		}
				
		@graffiti = DB::getNextRow_Graffiti($sth);
	
		DB::closeOracleConnection( $dbh );

		# Voting Handling Code
		$VoteForm = <<"EOF";
					<b>Thank you for voting!</b><br><BR>
					
					Total number of votes for this artwork:&nbsp;&nbsp;<b>$count</b>
EOF
	}

	# Set up Table for Voting
	$dc->q->param('VoteForm', $VoteForm);

	#################################################
	# Handle authorizing feature
	if ($IN::feature eq "accept")
	{
		# Make the change to the Database
		($dbh, $errstr) = DB::openOracleConnection();
		if( !$dbh )
		{
			$dc->error("[graffiti] Error connecting to database: $errstr", $errorURL);
		}
	
		# Update the Download Count
		($errN, $errstr)       = DB::modifyRow_GraffitiByUK( $dbh, $dc->DBaseTable('GRAFFITI'), $UniqKey, $graffiti[$dc->GraffitiOff('USERID')], $graffiti[$dc->GraffitiOff('SIZE')], $graffiti[$dc->GraffitiOff('TITLE')], $graffiti[$dc->GraffitiOff('DESC')], $graffiti[$dc->GraffitiOff('COMMENT')], $graffiti[$dc->GraffitiOff('DLCOUNT')], $graffiti[$dc->GraffitiOff('RANKTOTAL')], $graffiti[$dc->GraffitiOff('RANKCNT')], $graffiti[$dc->GraffitiOff('CURRANK')], 'N', $adminkey, time, $graffiti[$dc->GraffitiOff('VMS')] );
		if ($errN)
		{
			$dc->error("[graffiti] Updating Graffiti Information: $errN .... $errstr", $errorURL);
		}

		# Re-update the Graffiti Information
		($errN, $errstr, $sth) = DB::getRow_GraffitiByUK( $dbh, $dc->DBaseTable('GRAFFITI'), $UniqKey );
		if ($errN)
		{
			$dc->error("[graffiti] Unable to locate the particular Graffiti record: $errN .... $errstr", $errorURL);
		}
				
		@graffiti = DB::getNextRow_Graffiti($sth);
	
		DB::closeOracleConnection( $dbh );
	}
	elsif ($IN::feature eq "delete")
	{
		# Make the change to the Database
		($dbh, $errstr) = DB::openOracleConnection();
		if( !$dbh )
		{
			$dc->error("[graffiti] Error connecting to database: $errstr", $errorURL);
		}
	
		# Update the Download Count
		($errN, $errstr)       = DB::deleteRow_GraffitiByUK( $dbh, $dc->DBaseTable('GRAFFITI'), $UniqKey );
		if ($errN)
		{
			$dc->error("[graffiti] Delte Graffiti: $errN .... $errstr", $errorURL);
		}

		DB::closeOracleConnection( $dbh );

		# Output the delete page
		$dc->q->param('header', "Graffiti Deleted");
		$dc->q->param('message', "Graffiti delection has successfully completed.<BR>Please click on back to return to menu");
		$dc->output_form("html/graffiti_delete.html");
		exit(0);
	}

	###################################################
	# Handle displaying output

	if ($dc->Flags($user[$dc->UserOff('COUNTRY')]) ne "")
	{
		$dc->q->param('FlagImg', "<img src=\"/images/flags/".$dc->Flags($user[$dc->UserOff('COUNTRY')])."\" width=40 height=20 border=0>\n");	
	} else {
		# Draw the generic flag
		$dc->q->param('FlagImg', "<img src=\"/images/flags/flag_other.gif\" width=40 height=20 border=0>\n");
	}

	$dc->q->param('NAME', $user[$dc->UserOff('NAME')]);
	$location = $user[$dc->UserOff('CITY')].", ";
	if (($user[$dc->UserOff('COUNTRY')] eq "US") && ($user[$dc->UserOff('STATE')] ne "")) {
		$location .= $dc->state_name($user[$dc->UserOff('STATE')]).", ";
	}
	$location .= $dc->country_name($user[$dc->UserOff('COUNTRY')])."<br>\n";
	$dc->q->param('location', $location);

	# Graffiti Specific Information
	$dc->q->param('TITLE', $graffiti[$dc->GraffitiOff('TITLE')]);
	$dc->q->param('DESCR', $graffiti[$dc->GraffitiOff('DESC')]);
	$dc->q->param('COMMENT', $graffiti[$dc->GraffitiOff('COMMENT')]."<BR>");
	$dc->q->param('DLCount', $graffiti[$dc->GraffitiOff('DLCOUNT')]);

	# Convert the date into months days and seconds.
	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($graffiti[$dc->GraffitiOff('CDATE')]);
	$mon++;
	$year = ($year < 20) ? $year+2000 : $year+1900;
	$datestr = sprintf ("%02d/%02d/%04d",$mon,$mday,$year);

	$dc->q->param('DateC', $datestr);

	if ($user[$dc->UserOff('MFLAG')] eq 'Y')
	{
		$dc->q->param('email', "<A HREF=\"MAILTO:".$user[$dc->UserOff('MFLAG')]."\"><img src=\"/images/button_email.gif\" width=72 height=14 border=0 alt=\"\"></a>");
	}
	else
	{
		$dc->q->param('email', "");
	}

	$dc->q->param('RankImg', $RankTbl[(ceil($graffiti[$dc->GraffitiOff('CURRANK')]))]->[1]);

	if ($dc->file_exists("${path}/${UniqKey}.jpg") == 0)	# Create the file if it doesn't exist
	{
		# Create the graphic locally, otherwise don't bother
		@BINARY = unpack("C*", $graffiti[$dc->GraffitiOff('VMS')]);
		%graffitiInfo = $dc->vmfile_graffiti(\@BINARY);

		# Generate a new JPG image
		$image=Image::Magick->new(magick=>'jpg');

		$err = $image->BlobToImage($graffitiInfo{'image'});
		if ($err ne "")
		{
			$dc->error("[graffiti] Unable to convert blob to image. \"$err\".", $errorURL);
		}

		my $result = open( JPGFILE, ">${path}/${UniqKey}.jpg" );
		if( $result )
		{
			no strict;
			$err = $image->Write(file=>JPGFILE, filename=>"${path}/${UniqKey}.jpg",magick=>'jpg');
			if ($err ne "")
			{
				$dc->error("[graffiti] Unable to create JPG file.<BR>Graffiti file may be damaged.<BR>Please try again.", $errorURL);
			}
			strict;

			close( JPGFILE );
			chmod(0666,"${path}/${UniqKey}.jpg");
		}
		else
		{
			$dc->error("[graffiti] Unable to create thumbnail JPG file.<BR>Graffiti file may be damaged.<BR>Please try again.", $errorURL);
		}

		undef $image;		# Free up image memory
	}
	
	$dc->q->param('GraffitiImg', "/${servername}/${UniqKey}.jpg");

	# Administrator Page
	# Check if user has administator Rights
	if ($pending eq 'Y')
	{
		if ($graffiti[$dc->GraffitiOff('PENDING')] eq 'Y')
		{
			$dc->q->param('Pending', "<font color=\"red\"><b>-PENDING-</b></font>\n");

			if ($IN::feature eq "reject")
			{
				# Approval Page
				$ApprovalForm = <<"EOF";
			<table width=100% cellpadding=6 cellspacing=0 border=0>
				<tr>
					<td valign=top>
<b>-- Confirmation of Delete --</b><br>
Are you sure you want to perminately delete this graffiti?
										
					</td>
					<td valign=top align=center>
<form action="/cgi-bin/jetsetradio/graffiti.cgi">
EOF

				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"SIZE\" VALUE=\"".$IN::SIZE."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"MODE\" VALUE=\"".$IN::MODE."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"mailid\" VALUE=\"".$IN::mailid."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"type\" VALUE=\"".$IN::type."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"sort\" VALUE=\"".$IN::sort."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"rating\" VALUE=\"".$IN::rating."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"sortby\" VALUE=\"".$IN::sortby."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"title\" VALUE=\"".$IN::title."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"start\" VALUE=\"".$IN::start."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"max\" VALUE=\"".$IN::max."\">\n";

				$ApprovalForm .= <<"EOF";
<INPUT TYPE="hidden" NAME="UniqKey" VALUE="$UniqKey">
<INPUT TYPE="hidden" NAME="rate" VALUE="6">
<INPUT TYPE="hidden" NAME="feature" VALUE="delete">
<input type="Submit" name="Accept" value="Yes">
<input type="hidden" name="command" value="display">
</form>
					</td>
					<td valign=top align=right>
<form action="/cgi-bin/jetsetradio/graffiti.cgi">
EOF

				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"SIZE\" VALUE=\"".$IN::SIZE."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"MODE\" VALUE=\"".$IN::MODE."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"mailid\" VALUE=\"".$IN::mailid."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"type\" VALUE=\"".$IN::type."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"sort\" VALUE=\"".$IN::sort."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"rating\" VALUE=\"".$IN::rating."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"sortby\" VALUE=\"".$IN::sortby."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"title\" VALUE=\"".$IN::title."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"start\" VALUE=\"".$IN::start."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"max\" VALUE=\"".$IN::max."\">\n";

				$ApprovalForm .= <<"EOF";
<INPUT TYPE="hidden" NAME="UniqKey" VALUE="$UniqKey">
<INPUT TYPE="hidden" NAME="rate" VALUE="6">
<INPUT TYPE="hidden" NAME="feature" VALUE="">
<input type="Submit" name="Reject" value="No">
<input type="hidden" name="command" value="display">
</form>
					</td>
				</tr>
			</table>
EOF
			}
			else
			{
				# Approval Page
				$ApprovalForm = <<"EOF";
			<table width=100% cellpadding=6 cellspacing=0 border=0>
				<tr>
					<td valign=top>
<b>-- Awaiting approval --</b><br>
Select one of the following options:
										
					</td>
					<td valign=top align=center>
<form action="/cgi-bin/jetsetradio/graffiti.cgi">
EOF

				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"SIZE\" VALUE=\"".$IN::SIZE."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"MODE\" VALUE=\"".$IN::MODE."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"mailid\" VALUE=\"".$IN::mailid."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"type\" VALUE=\"".$IN::type."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"sort\" VALUE=\"".$IN::sort."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"rating\" VALUE=\"".$IN::rating."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"sortby\" VALUE=\"".$IN::sortby."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"title\" VALUE=\"".$IN::title."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"start\" VALUE=\"".$IN::start."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"max\" VALUE=\"".$IN::max."\">\n";

				$ApprovalForm .= <<"EOF";
<INPUT TYPE="hidden" NAME="UniqKey" VALUE="$UniqKey">
<INPUT TYPE="hidden" NAME="rate" VALUE="6">
<INPUT TYPE="hidden" NAME="feature" VALUE="accept">
<input type="Submit" name="Accept" value="Authorize">
<input type="hidden" name="command" value="display">
</form>
					</td>
					<td valign=top align=right>
<form action="/cgi-bin/jetsetradio/graffiti.cgi">
EOF

				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"SIZE\" VALUE=\"".$IN::SIZE."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"MODE\" VALUE=\"".$IN::MODE."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"mailid\" VALUE=\"".$IN::mailid."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"type\" VALUE=\"".$IN::type."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"sort\" VALUE=\"".$IN::sort."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"rating\" VALUE=\"".$IN::rating."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"sortby\" VALUE=\"".$IN::sortby."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"title\" VALUE=\"".$IN::title."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"start\" VALUE=\"".$IN::start."\">\n";
				$ApprovalForm .= "<INPUT TYPE=\"hidden\" NAME=\"max\" VALUE=\"".$IN::max."\">\n";

				$ApprovalForm .= <<"EOF";
<INPUT TYPE="hidden" NAME="UniqKey" VALUE="$UniqKey">
<INPUT TYPE="hidden" NAME="rate" VALUE="6">
<INPUT TYPE="hidden" NAME="feature" VALUE="reject">
<input type="Submit" name="Reject" value="Delete">
<input type="hidden" name="command" value="display">
</form>
					</td>
				</tr>
			</table>
EOF
			}
		}
		else
		{
			# Read the administrator's information
			($dbh, $errstr) = DB::openOracleConnection();
			if( !$dbh )
			{
				$dc->error("[graffiti] Error connecting to database: $errstr", $errorURL);
			}
	
			($errN, $errstr, @admin) = DB::getRow_UserByUK( $dbh, $dc->DBaseTable('USER'), $graffiti[$dc->GraffitiOff('ADMINID')] );
			if ($errN)
			{
				$dc->error("[graffiti] Fetching admin user key: $errN .... $errstr", $errorURL);
			}
		
			DB::closeOracleConnection( $dbh );

			# Convert the date into months days and seconds.
			($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($graffiti[$dc->GraffitiOff('ADATE')]);
			$mon++;
			$year = ($year < 20) ? $year+2000 : $year+1900;
			$datestr = sprintf ("%02d/%02d/%04d",$mon,$mday,$year);

			# Approval Page
			$ApprovalForm = <<"EOF";
			<table width=100% cellpadding=6 cellspacing=0 border=0>
				<tr>
					<td>
<b>-- Graffiti approved --</b><br>
					
Approved: $datestr<br>
					
by: $admin[$dc->UserOff('NAME')]&nbsp;&nbsp;<A HREF="MAILTO:$admin[$dc->UserOff('EMAIL')]"><img src="/images/button_email.gif" width=72 height=14 border=0 alt=""></a>
					
					</td>
					<td align=right>
					</td>
				</tr>
			</table>
EOF
		}


		$dc->q->param('ApprovalForm', $ApprovalForm);
	}
	else
	{
		$dc->q->param('Pending', "");
		$dc->q->param('ApprovalForm', "");
	}

	###############################################
	# Handle features like voting and authorization

}

#--------------------------------------------------------
# Data Check
#--------------------------------------------------------
sub check_search {
	my ($errN, $errstr, $dbh, $sth, @row, @graffiti);
	my ($searchtime, $count, $range);

	$searchtime = 0;

	# Determine if user has Administration Rights
	# Read user information from the table
	if ($userid ne "")
	{
		($dbh, $errstr) = DB::openOracleConnection();
		if( !$dbh )
		{
			$dc->error("[graffiti] Error connecting to database: $errstr", $errorURL);
		}
		
		($errN, $errstr, @row) = DB::getRow_UserByUK( $dbh, $dc->DBaseTable('USER'), $userid );
		if ($errN)
		{
			$dc->error("[graffiti] Fetching user key: $errN .... $errstr", $errorURL);
		}
		
		DB::closeOracleConnection( $dbh );

		if ($row[$dc->UserOff('ADMIN')] eq 'Y')
		{
			$adminkey = $userid;
			$pending = 'Y';
		}
	}
		
	if ($Mode == 0)
	{
		# Define that only 10 records will be shown

		# Define same parameters as in Advanced Search
		# Set up as Top Downloaded
		$dc->q->param('type', "ALL");
		$dc->q->param('mailid', "");
		$dc->q->param('start', 1);
		$dc->q->param('sort', "download");
		$dc->q->param('rating', -1);
		$dc->q->param('sortby', "newest");
		$dc->q->param('title', "");
	}
	elsif ($Mode == 1)
	{
		# Define same parameters as in Advanced Search
		# Set up as Top Rate
		$dc->q->param('type', "ALL");
		$dc->q->param('mailid', "");
		$dc->q->param('start', 1);
		$dc->q->param('sort', "rated");
		$dc->q->param('rating', -1);
		$dc->q->param('sortby', "newest");
		$dc->q->param('title', "");
	}
	elsif ($Mode == 2)
	{
		# Define that only 10 records will be shown

		# Define same parameters as in Advanced Search
		# Set up as Most Recent Downloads
		$dc->q->param('type', "ALL");
		$dc->q->param('mailid', "");
		$dc->q->param('start', 1);
		$dc->q->param('sort', "date");
		$dc->q->param('rating', -1);
		$dc->q->param('sortby', "newest");
		$dc->q->param('title', "");
	}
	else
	{
		# Use the Advanced Search Options
		$dc->q->param('type', $IN::type);
		$dc->q->param('mailid', $IN::mailid);			# Clear the value for now!
		$dc->q->param('start', $IN::start);
		$dc->q->param('sort', $IN::sort);
		$dc->q->param('rating', $IN::rating);
		$dc->q->param('sortby', $IN::sortby);
		$dc->q->param('title', $IN::title);
		$dc->q->param('max', $IN::max);
	}
		
	# Either today's or All time (5 Years)
	if ($dc->q->param('type') eq "TODAY")
	{
		$searchtime = (1*24*60*60);			# (24 hours in seconds)
		$dc->q->param('type', "TODAY");
	}
	else
	{
		# 5 Years in seconds
		$searchtime = (5*365*24*60*60);		# (5 Years in seconds)
		$dc->q->param('type', "ALL");
	}

	$dc->q->param('message', "You can search for Graffiti here. Search by Username or go through all the graffiti.  ");

	if($dc->q->param('max') ne "") {
		$MAX = $dc->q->param('max');
		if($MAX<=0) {
			# Set default maximum size
			$MAX=10;
		}
	}
		
	$START=0;
	if($dc->q->param('start') ne "") {
		$START = ($dc->q->param('start') - 1);
		if($START<0) {
			$START=0;
		}
	}

	#Initialize base values
	$dc->q->param('max', $MAX);
	$dc->q->param('start', $START + 1);

	# Sort parameters has not been initialized
	if ($dc->q->param('sort') eq "")
	{
		$dc->q->param('sort', "rated");
	}

	if ($dc->q->param('rating') eq "")
	{
		$dc->q->param('rating', -1);
	}

	if ($dc->q->param('sortby') eq "")
	{
		$dc->q->param('sortby', "newest");
	}

	# Now determine types of searches
	if ($dc->q->param('sort') eq "rated")
	{
		$sort = 0;		# Top Rated
		$sortby = 0;	# Not Used
		if ($dc->q->param('rating') == 1)
		{
			$RankLow = 0.5;
			$RankHigh = (1.5 - 0.0000001);
		}
		elsif ($dc->q->param('rating') == 2)
		{
			$RankLow = 1.5;
			$RankHigh = (2.5 - 0.0000001);
		}
		elsif ($dc->q->param('rating') == 3)
		{
			$RankLow = 2.5;
			$RankHigh = (3.5 - 0.0000001);
		}
		elsif ($dc->q->param('rating') == 4)
		{
			$RankLow = 3.5;
			$RankHigh = (4.5 - 0.0000001);
		}
		elsif ($dc->q->param('rating') == 5)
		{
			$RankLow = 4.5;
			$RankHigh = (5.5 - 0.0000001);
		}
		elsif ($dc->q->param('rating') == 6)
		{
			$RankLow = 5.5;
			$RankHigh = 6.0;
		}
		else
		{
			$sortby = 1;	# Use standard search
			$RankLow = 0.0;
			$RankHigh = 6.0;
		}
	}
	elsif ($dc->q->param('sort') eq "download")
	{
		$sort = 1;		# Top Downloads
		$sortby = -1;	# Not Used
	}
	elsif ($dc->q->param('sort') eq "date")
	{
		$sort = 2;		# Order by Date
		if ($dc->q->param('sortby') eq "newest")
		{
			$sortby = 0;	# Sort by Newest (Descending)
		}
		elsif ($dc->q->param('sortby') eq "oldest")
		{
			$sortby = 1;	# Sort by Oldest (Ascending)
		}
		elsif ($dc->q->param('sortby') eq "title")
		{
			$sortby = 2;	# Sort by Alphabetical Order
		}
	}
	
	# Title search over rides all other searches
	if ($dc->q->param('title') ne "")
	{
		$sort = 3;		# Search by Title
		$sortby = -1;	# Not Used
		$title = $dc->q->param('title');
	}

	# Save original Maximum Record Score
	my $OldMax = $MaxRec;

	# Determine total number of records for All time and Today
	&determine_rank($Size, "ALL");
	$dc->q->param("maxall", $MaxRec);
	&determine_rank($Size, "TODAY");
	$dc->q->param("maxtoday", $MaxRec);

	$MaxRec = $OldMax;

	if($IN::mailid ne "") {
		$searchID = $dc->get_key($IN::mailid);
		if ($searchID eq "")
		{
			$dc->q->param('message', "<font color=\"#ff0000\">Sorry, no matches were found containing <b>$IN::mailid</b>.</font><p>Please try again. ");
		
			# Terminate the search
			$dc->output_form("html/graffiti_search.html");
			undef($searchID);
			exit(0);
		}
		else
		{
			($dbh, $errstr) = DB::openOracleConnection();
			if( !$dbh )
			{
				$dc->error("[graffiti] Error connecting to database: $errstr", $errorURL);
			}

			($errN, $errstr, $sth) = DB::getRow_GraffitiByUserIDandSize( $dbh, $dc->DBaseTable('GRAFFITI'), $searchID, $Size, $pending, "DESC", $searchtime );
			if ($errN)
			{
				$dc->error("[graffiti] Unable to Search for particular User Graffiti: $errN .... $errstr", $errorURL);
			}

			$count = 0;
			while( (@graffiti) = DB::getNextRow_Graffiti($sth) )
			{
				if ($graffiti[$dc->GraffitiOff('USERID')] eq $searchID)
				{
					++$count;		# Entry found so go for it
				}
			}

			DB::closeOracleConnection( $dbh );

			if ($count == 0)
			{
				$dc->q->param('message', "<font color=\"#ff0000\">Sorry, no matches were found for User \"<b>$IN::mailid</b>\".</font><p>Please try again. ");

				# Terminate the search
				$dc->output_form("html/graffiti_search.html");
				undef($UserKey);
				exit(0);
			}
		}
	}
}

#--------------------------------------------------------
# Read Key Data
#--------------------------------------------------------
sub read_data {
	my $key = shift;
	my ($errN, $errstr, $dbh, $sth, @row);

	($dbh, $errstr) = DB::openOracleConnection();
	if( !$dbh )
	{
		$dc->error("[graffiti] Error connecting to database: $errstr", $errorURL);
	}

	# Calculate the maximum number of records
	&determine_rank($Size, "ALL");
	
	if($START < $MaxRec) {
		if(($MaxRec-$START)>($MAX)) {
			$END=$START+$MAX;
			$nextflag=0; 
		} else {
			$END=$MaxRec;
			$nextflag=1;
		}
	}

	return $dbh;
}

#--------------------------------------------------------
# HTML Output
#--------------------------------------------------------
sub output_records {
	my $dbh = shift;

	my ($errN, $errstr, $sth, @user, @graffiti, $UserID);
	my $loop = 0;
	my $searchtime = 0;

    print "Content-type: text/html\n";
    print <<EOF;

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">

<html>
<head>
	<title>Jet Grind Radio</title>
<meta name=x-uirequest content=nohscroll>
<meta HTTP-EQUIV="PRAGMA" CONTENT="NO-CACHE">
</head>

<body bgcolor=#E6CE00 x-marginleft=0 TEXT="#660000" LINK="#B96800" VLINK="#B96800" ALINK="#B96800" marginheight=0 marginwidth=0 topmargin=0 leftmargin=0>


<center>

<TABLE WIDTH="550" BORDER=0 CELLPADDING=0 CELLSPACING=0>
	<tr>
		<TD valign=top width=550><img src="/images/title_graffiti.gif" width=550 height=46 border=0 alt=""></td>
	</tr>
</table>
<TABLE WIDTH="550" BORDER=0 CELLPADDING=0 CELLSPACING=0>
	<tr>
		<TD width=7 valign=top><img src="/images/clear.gif" width=7 height=1 border=0 alt=""></td>
		<TD width=368 valign=top>
		
Click on Thumbnails to view art.
		<br><br>
EOF

	&output_link;

	print "<br>";
	
	if ($dc->q->param('type') eq "TODAY")
	{
		$searchtime = (1*24*60*60);
	}
	else
	{
		# 5 Years in seconds
		$searchtime = (5*365*24*60*60);
	}

	if($IN::mailid ne "") {
		# Only searching for titles by a specific user
		# Only display the results by showing from Newest to Oldest

		$UserID = $dc->get_key($IN::mailid);

		($errN, $errstr, $sth) = DB::getRow_GraffitiByUserIDandSize( $dbh, $dc->DBaseTable('GRAFFITI'), $UserID, $Size, $pending, "DESC", $searchtime );
		if ($errN)
		{
			$dc->error("[graffiti] Unable to get User Graffiti: $errN .... $errstr", $errorURL);
		}
	} else {
		# General search so use the different queries

		# Following code handles the various types of searches

		if ($sort == 0)
		{
			if ($sortby == 0)
			{
				# Search by Ratings
				($errN, $errstr, $sth) = DB::getRow_GraffitiByRankandSize( $dbh, $dc->DBaseTable('GRAFFITI'), $RankLow, $RankHigh, $Size, $pending, "DESC", $searchtime );
				if ($errN)
				{
					$dc->error("[graffiti] Unable to get Graffiti Rank and Size Ascending Events: $errN .... $errstr", $errorURL);
				}
			}
			elsif ($sortby == 1)
			{
				# Just sort by Rank
				($errN, $errstr, $sth) = DB::getRow_GraffitiBySizeOrderbyRank( $dbh, $dc->DBaseTable('GRAFFITI'), $Size, $pending, "DESC", $searchtime );
				if ($errN)
				{
					$dc->error("[graffiti] Unable to get Graffiti DL Count and Size Descending Events: $errN .... $errstr", $errorURL);
				}
			}
			else
			{
				# Something went wrong so use the default search
				($errN, $errstr, $sth) = DB::getRow_GraffitiBySize( $dbh, $dc->DBaseTable('GRAFFITI'), $Size, $pending, "ASC", $searchtime );
				if ($errN)
				{
					$dc->error("[graffiti] Unable to get Graffiti Size Ascending Events: $errN .... $errstr", $errorURL);
				}
			}
		}
		elsif ($sort == 1)
		{
			# Search by Top Downloads
			($errN, $errstr, $sth) = DB::getRow_GraffitiByDLCountandSize( $dbh, $dc->DBaseTable('GRAFFITI'), $Size, $pending, "DESC", $searchtime );
			if ($errN)
			{
				$dc->error("[graffiti] Unable to get Graffiti DL Count and Size Descending Events: $errN .... $errstr", $errorURL);
			}
		}
		elsif ($sort == 2)
		{
			# Search by Newest Date
			if ($sortby == 0)
			{
				($errN, $errstr, $sth) = DB::getRow_GraffitiBySize( $dbh, $dc->DBaseTable('GRAFFITI'), $Size, $pending, "DESC", $searchtime );
				if ($errN)
				{
					$dc->error("[graffiti] Unable to get Graffiti Size and Ascending Events: $errN .... $errstr", $errorURL);
				}
			}
			# Search by Oldest Date
			elsif ($sortby == 1)
			{
				($errN, $errstr, $sth) = DB::getRow_GraffitiBySize( $dbh, $dc->DBaseTable('GRAFFITI'), $Size, $pending, "ASC", $searchtime );
				if ($errN)
				{
					$dc->error("[graffiti] Unable to get Graffiti Size and Descending Events: $errN .... $errstr", $errorURL);
				}
			}
			# Sort by Alphabetical Order
			elsif ($sortby == 2)
			{
				($errN, $errstr, $sth) = DB::getRow_GraffitiBySizeOrderbyTitle( $dbh, $dc->DBaseTable('GRAFFITI'), $Size, $pending, "ASC", $searchtime );
				if ($errN)
				{
					$dc->error("[graffiti] Unable to get Graffiti Title and Size Ascending Events: $errN .... $errstr", $errorURL);
				}
			}
			else
			{
				# Something went wrong so use the default search
				($errN, $errstr, $sth) = DB::getRow_GraffitiBySize( $dbh, $dc->DBaseTable('GRAFFITI'), $Size, $pending, "ASC", $searchtime );
				if ($errN)
				{
					$dc->error("[graffiti] Unable to get Graffiti Size Ascending Events: $errN .... $errstr", $errorURL);
				}
			}
		}
		elsif ($sort == 3)
		{
			# Search by Title
			($errN, $errstr, $sth) = DB::getRow_GraffitiSearchTitleandSize( $dbh, $dc->DBaseTable('GRAFFITI'), $title, $Size, $pending, "ASC", $searchtime );
			if ($errN)
			{
				$dc->error("[graffiti] Unable to get Graffiti Title and Size Ascending Events: $errN .... $errstr", $errorURL);
			}
		}
		else
		{
			# Something went wrong so use the default search
			($errN, $errstr, $sth) = DB::getRow_GraffitiBySize( $dbh, $dc->DBaseTable('GRAFFITI'), $Size, $pending, "ASC", $searchtime );
			if ($errN)
			{
				$dc->error("[graffiti] Unable to get Graffiti Size Ascending Events: $errN .... $errstr", $errorURL);
			}
		}
	}

	for($loop=0;$loop<$END;$loop++) {
		(@graffiti) = DB::getNextRow_Graffiti($sth);
		if ($loop >= $START)
		{
			($errN, $errstr, @user) = DB::getRow_UserByUK( $dbh, $dc->DBaseTable('USER'), $graffiti[$dc->GraffitiOff('USERID')] );
			if ($errN)
			{
				$dc->error("[graffiti] Fetching user info: $errN .... $errstr", $errorURL);
			}
			&build_table($loop, \@user, \@graffiti);
		}
	}

	DB::closeOracleConnection( $dbh );

	&output_link;

    print <<EOF;
<!-- end of content -->
		<br><br>


<form action="/cgi-bin/jetsetradio/graffiti.cgi">
<INPUT TYPE="hidden" NAME="SIZE" VALUE="$IN::SIZE">
<INPUT TYPE="hidden" NAME="MODE" VALUE="$IN::MODE">
<INPUT TYPE="hidden" NAME="mailid" VALUE="$IN::mailid">
<INPUT TYPE="hidden" NAME="type" VALUE="$IN::type">
<INPUT TYPE="hidden" NAME="sort" VALUE="$IN::sort">
<INPUT TYPE="hidden" NAME="rating" VALUE="$IN::rating">
<INPUT TYPE="hidden" NAME="sortby" VALUE="$IN::sortby">
<INPUT TYPE="hidden" NAME="title" VALUE="$IN::title">
<INPUT TYPE="hidden" NAME="start" VALUE="$IN::start">
<input type="hidden" name="max" value="$IN::max">
<input type="hidden" name="command" value="menu">
<INPUT TYPE="Image" src="/images/button_back.gif" width=200 height=30 border=1 alt="" align=right>
</form><br></td>
		<TD width=175 valign=top><img src="/images/graphic_graffiti.gif" width=175 height=400 border=0 alt=""><br><br></td>
	</tr>
</table>
</body>

</html>
EOF

}

sub build_table {
	my $rank = shift;
	my $user = shift;
	my $graffiti = shift;

	my $PlayerName = "";
	my ($date, $datestr, $UniqKey, @BINARY, %graffitiInfo, $image, $err, $result);

	$UniqKey = $graffiti->[$dc->GraffitiOff('KEY')];

	if ($dc->file_exists("${path}/${UniqKey}.jpg") == 0)	# Create the file if it doesn't exist
	{
		# Create the graphic locally, otherwise don't bother
		@BINARY = unpack("C*", $graffiti->[$dc->GraffitiOff('VMS')]);
		%graffitiInfo = $dc->vmfile_graffiti(\@BINARY);

		# Generate a new JPG image
		$image=Image::Magick->new(magick=>'jpg');

		$err = $image->BlobToImage($graffitiInfo{'image'});
		if ($err ne "")
		{
			$dc->error("[graffiti] Unable to convert blob to image. \"$err\".", $errorURL);
		}

		my $result = open( JPGFILE, ">${path}/${UniqKey}.jpg" );
		if( $result )
		{
			no strict;
			$err = $image->Write(file=>JPGFILE, filename=>"${path}/${UniqKey}.jpg",magick=>'jpg');
			if ($err ne "")
			{
				$dc->error("[graffiti] Unable to create JPG file.<BR>Graffiti file may be damaged.<BR>Please try again.", $errorURL);
			}
			strict;

			close( JPGFILE );
			chmod(0666,"${path}/${UniqKey}.jpg");
		}
		else
		{
			$dc->error("[graffiti] Unable to create thumbnail JPG file.<BR>Graffiti file may be damaged.<BR>Please try again.", $errorURL);
		}

		undef $image;		# Free up image memory
	}

	if ($dc->file_exists("${path}/${UniqKey}_sm.jpg") == 0)	# Create the file if it doesn't exist
	{
		$image=Image::Magick->new(magick=>'jpg');
		$err = $image->Read($path."/".$UniqKey.".jpg");
		if ($err ne "")
		{
			$dc->error("[graffiti] Unable to read small jpg. \"$err\".", $errorURL);
		}

		my $result = open( JPGFILE, ">${path}/${UniqKey}_sm.jpg" );
		if( $result )
		{
			if ($Size == 0)
			{
				$err = $image->Scale(width=>32, height=>32);
				if ($err ne "")
				{
					$dc->error("[graffiti] Unable to scale image. \"$err\".", $errorURL);
				}
			}
			elsif ($Size == 1)
			{
				$err = $image->Scale(width=>64, height=>32);
				if ($err ne "")
				{
					$dc->error("[graffiti] Unable to scale image. \"$err\".", $errorURL);
				}
			}
			elsif ($Size == 2)
			{
				$err = $image->Scale(width=>128, height=>32);
				if ($err ne "")
				{
					$dc->error("[graffiti] Unable to scale image. \"$err\".", $errorURL);
				}
			}

			no strict;
			$err = $image->Write(file=>JPGFILE, filename=>"${path}/${UniqKey}_sm.jpg",magick=>'jpg');
			if ($err ne "")
			{
				$dc->error("[graffiti] Unable to save scaled smaller image. \"$err\".", $errorURL);
			}
			strict;

			close( JPGFILE );
			chmod(0666,"${path}/${UniqKey}_sm.jpg");
		}
	}
	
	print <<EOF;
			<table width=360 cellpadding=5 cellspacing=0 border=0>
				<tr>
<!--BEGIN RANK TABLE-->
EOF

	print "<td width=210 bgcolor=#cccccc valign=top>";
	print $graffiti->[$dc->GraffitiOff('TITLE')];
	if ($graffiti->[$dc->GraffitiOff('PENDING')] eq 'Y')
	{
		print "&nbsp;&nbsp;<font color=\"red\"><b>-PENDING-</b></font>\n";
	}
	print "<BR>\n";

	if ($dc->Flags($user->[$dc->UserOff('COUNTRY')]) ne "")
	{
		print "<img src=\"/images/flags/".$dc->Flags($user->[$dc->UserOff('COUNTRY')])."\" width=40 height=20 border=0><BR>\n";	
	} else {
		# Draw the generic flag
		print "<img src=\"/images/flags/flag_other.gif\" width=40 height=20 border=0><BR>\n";
	}
	print "$user->[$dc->UserOff('NAME')]<br>\n";
	if ($user->[$dc->UserOff('CITY')] ne "")
	{
		# Don't bother printing the comma if no city is chosen
		print "$user->[$dc->UserOff('CITY')],<BR>";
	}
	if (($user->[$dc->UserOff('COUNTRY')] eq "US") && ($user->[$dc->UserOff('STATE')] ne "")) {
		printf "%s,<BR>", $dc->state_name($user->[$dc->UserOff('STATE')]);
	}
	printf "%s<br>\n", $dc->country_name($user->[$dc->UserOff('COUNTRY')]);
	print "</td>\n";

	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($graffiti->[$dc->GraffitiOff('CDATE')]);
	$mon++;
	$year = ($year < 20) ? $year+2000 : $year+1900;
	$datestr = sprintf ("%02d/%02d/%04d",$mon,$mday,$year);

	print "<td width=150 bgcolor=#cccccc valign=top>";
	print "<FORM METHOD=\"POST\" ACTION=\"/cgi-bin/jetsetradio/graffiti.cgi\">";
	print $datestr;
	print "<INPUT TYPE=\"hidden\" NAME=\"SIZE\" VALUE=\"".$dc->q->param('SIZE')."\">\n";
	print "<INPUT TYPE=\"hidden\" NAME=\"MODE\" VALUE=\"".$dc->q->param('MODE')."\">\n";
	print "<INPUT TYPE=\"hidden\" NAME=\"mailid\" VALUE=\"".$dc->q->param('mailid')."\">\n";
	print "<INPUT TYPE=\"hidden\" NAME=\"type\" VALUE=\"".$dc->q->param('type')."\">\n";
	print "<INPUT TYPE=\"hidden\" NAME=\"sort\" VALUE=\"".$dc->q->param('sort')."\">\n";
	print "<INPUT TYPE=\"hidden\" NAME=\"rating\" VALUE=\"".$dc->q->param('rating')."\">\n";
	print "<INPUT TYPE=\"hidden\" NAME=\"sortby\" VALUE=\"".$dc->q->param('sortby')."\">\n";
	print "<INPUT TYPE=\"hidden\" NAME=\"title\" VALUE=\"".$dc->q->param('title')."\">\n";
	print "<INPUT TYPE=\"hidden\" NAME=\"start\" VALUE=\"".$dc->q->param('start')."\">\n";
	print "<input type=\"hidden\" name=\"max\" value=\"".$dc->q->param('max')."\">\n";

	print "<INPUT TYPE=\"hidden\" NAME=\"UniqKey\" VALUE=\"$graffiti->[$dc->GraffitiOff('KEY')]\">\n";
	print "<INPUT TYPE=\"hidden\" NAME=\"rate\" VALUE=\"6\">\n";
	print "<INPUT TYPE=\"hidden\" NAME=\"feature\" VALUE=\"\">\n";
	print "<INPUT TYPE=\"hidden\" NAME=\"command\" VALUE=\"display\">\n";
	print "<INPUT TYPE=\"image\" src=\"/${servername}/".$graffiti->[$dc->GraffitiOff('KEY')]."_sm.jpg\"";
	if ($Size == 0)
	{
		print " width=32";
	}
	elsif ($Size == 1)
	{
		print " width=64";
	}
	elsif ($Size == 2)
	{
		print " width=128";
	}
	print " height=32 border=0>";

	print "<BR>Rating:&nbsp;".$RankTbl[(ceil($graffiti->[$dc->GraffitiOff('CURRANK')]))]->[0]."<br>\n";

	print "Downloads: ".$graffiti->[$dc->GraffitiOff('DLCOUNT')]."\n";
	print "</FORM>";

	print <<EOF;
			</td>
		</tr>
	</table><br>
<!--END RANK TABLE-->	
EOF
}

sub output_link {
	# Link Menu Output
	my $str = "";
	my $prank = 0;
	my $nrank = 0;
		
	if($START > 0) {
		$prank=$START-$MAX+1;
		if($prank<0) { $prank=0; }
		$str =	"SIZE=".$dc->q->param('SIZE')."&MODE=".$dc->q->param('MODE')."&command=view".
				"&mailid=".$dc->q->param('mailid')."&type=".$dc->q->param('type').
				"&sort=".$dc->q->param('sort')."&rating=".$dc->q->param('rating').
				"&sortby=".$dc->q->param('sortby')."&title=".$dc->q->param('title').
				"&start=".$prank."&max=".$MAX;
		print "<a href=\"$cgiPath?$str\"><img src=\"/images/button_prev.gif\" width=70 height=14 border=0 alt=\"\"></a>";
	}
	else
	{
		print "<img src=\"/images/clear.gif\" width=70 height=14 border=0 alt=\"\">";
	}

	print "<img src=\"/images/clear.gif\" width=210 height=14 border=0 alt=\"\">";
	if($nextflag==0) {
		$nrank=$START+$MAX+1;
		$str =	"SIZE=".$dc->q->param('SIZE')."&MODE=".$dc->q->param('MODE')."&command=view".
				"&mailid=".$dc->q->param('mailid')."&type=".$dc->q->param('type').
				"&sort=".$dc->q->param('sort')."&rating=".$dc->q->param('rating').
				"&sortby=".$dc->q->param('sortby')."&title=".$dc->q->param('title').
				"&start=".$nrank."&max=".$MAX;
		print "<a href=\"$cgiPath?$str\"><img src=\"/images/button_next.gif\" width=70 height=14 border=0 alt=\"\"></a>";
	}
	else
	{
		print "<img src=\"/images/clear.gif\" width=70 height=14 border=0 alt=\"\"></a>";
	}
}

#------------------------ EOF --------------------------
